#coding: utf-8

import os,platform,json,time,zipfile,shutil,socket
import logging, logging.handlers

logger = None
curUser = None
if os.sep == '/':
	TMP_PATH = '/tmp'
else:
	TMP_PATH = 'C:/Windows/Temp'

def getPkgNameInLog(logFile) -> str:
	'''find dy-client-*.deb, then return this name, or None if not found'''
	fp = open(logFile, 'rb')
	fp.seek(-10240, 2)
	txtBuf = fp.read(10240)
	fp.close()
	if b'dy-client-' not in txtBuf:
		logger.warning('not found: dy-client-xx')
		return None

	iBegin = txtBuf.find(b'dy-client-')
	iEnd = txtBuf.find(b'.deb', iBegin)
	if iEnd < 0 or iEnd-iBegin > 256:
		iEnd = txtBuf.find(b'.rpm', iBegin)
	if iEnd < 0 or iEnd-iBegin > 256:
		logger.warning('cannot find deb or rpm')
		return None
	return txtBuf[iBegin:iEnd].decode()

def findPackageInLocal() -> str:
	'''find dy-client-*.deb in local folderand return, or None if not found'''
	if curUser == 'root':
		cmd = 'find /root/ -name dy-client-*'
	else:
		cmd = 'find /home/%s -name dy-client-*' % curUser
	with os.popen(cmd) as pConsole:
		filePath = pConsole.read()
	if len(filePath) == 0:
		logger.warning('not found dy-client-*')
		return None
	i = filePath.rfind('/')
	return filePath[i+1:]

def updateConfig():
	'''首次运行时，更新客户端config.json ServerAddress,AutoLogon等字段'''
	CLIENT_PATH = '/opt/DYSJ/SEC/client'
	try:
		with open(CLIENT_PATH+'/config.json', 'r') as fp:
			jsConfig = json.load(fp)
	except Exception as e:
		logger.warning(repr(e))
		return -1
	if 'ConfigUpdated' in jsConfig:
		logger.info('ConfigUpdated')
		return 0

	logFile = '/opt/softmanager/tipterminal/var/log/httcgui.log_'+curUser+'_' + time.strftime("%Y-%m-%d", time.localtime())
	if os.path.exists(logFile):
		logger.info('found '+ logFile)
		pkgName = getPkgNameInLog(logFile)
	else:
		logger.info('find in local')
		pkgName = findPackageInLocal()
	if pkgName == None:
		return -2
	logger.info('package:'+ pkgName) # dy-client-IP+10.1.2.8-AutoLogon+3-AutoHide+1-arm64.deb

	for itm in  pkgName.split('-'):
		if '+' not in itm:
			continue
		kv = itm.split('+')
		if kv[0] == 'IP':
			kv[0] = 'ServerAddress'
			kv[1] += ':8085'
		logger.info('update:'+ kv[0]+' '+kv[1])
		if kv[1].isdigit():
			jsConfig[kv[0]] = int(kv[1])
		else:
			jsConfig[kv[0]] = kv[1]
		jsConfig['ConfigUpdated'] = 1

	if 'ConfigUpdated' in jsConfig:
		try:
			with open(CLIENT_PATH+'/config.json', 'w') as fp:
				json.dump(jsConfig, fp, indent=4)
			logger.info('config.json updated')
		except Exception as e:
			logger.warning(repr(e))
	else:
		logger.warning('config.json not updated')
	return 1

def updateClient():
	'''find /tmp/dy-client-yymmdd-all.zip, extract all then overwrite files '''
	zipFile = None
	for fname in os.listdir(TMP_PATH):
		if fname.startswith('dy-client') and fname.endswith('.zip'):
			zipFile = os.path.join(TMP_PATH, fname)
			break
	if zipFile == None:
		return

	logger.info('update pkg:'+ zipFile)
	osType = None
	if len(zipFile.split('-')) > 3:
		osType = zipFile.split('-')[3][:-4] # all,win32,aarch64,x86_64,mips64el
		if os.sep == '/' and osType != 'all' and osType != platform.machine():
			logger.warning(osType+ ' not for this machine')
			osType = None
		if os.sep == '\\' and osType != 'win32':
			logger.warning(osType+ ' not for win32')
			osType = None
	else:
		logger.warning('osType not found')
	if osType == None:
		os.remove(zipFile)
		return

	excPath = zipFile[:-4]
	try:
		if os.path.exists(excPath):
			shutil.rmtree(excPath)
		os.makedirs(excPath)
		fz = zipfile.ZipFile(zipFile, 'r')
		fz.extractall(excPath)
		fz.close()
	except Exception as e:
		logger.warning(repr(e))
		os.remove(zipFile)
		return
	os.remove(zipFile)

	upScript = excPath + '/preup.' + 'bat' if os.sep=='\\' else 'sh'
	if os.path.exists(upScript):
		try:
			os.system(upScript)
		except Exception as e:
			logger.warning(repr(e))

	for curPath, listDir, listFile in os.walk(excPath):
		if os.sep == '/': # /tmp/dy-client-xx/opt/DYSJ/SEC/client -> /opt/DYSJ/SEC/client
			dstPath = curPath.replace(excPath, '')
		else:	# C:/Windows/Temp/dy-client-xx -> C:/Windows/SecuPrint Driver
			dstPath = 'C:/Windows/SecuPrint Driver'
		if len(dstPath) < 3:
			continue
		if not os.path.exists(dstPath):
			logger.info('mkdir '+ dstPath)
			os.makedirs(dstPath)
		for fname in listFile:
			srcFile = os.path.join(curPath, fname)
			logger.info('update:'+ srcFile)
			shutil.copy(srcFile, dstPath)

	upScript = excPath + '/postup.' + 'bat' if os.sep=='\\' else 'sh'
	if os.path.exists(upScript):
		try:
			os.system(upScript)
		except Exception as e:
			logger.warning(repr(e))
	shutil.rmtree(excPath)

	# [7e 05 len 00 msg "客户端已升级到xxxx版本..."]
	try:
		sockTcp = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		sockTcp.connect(('127.0.0.1', 17002))
		cmdMsg = '客户端已升级，请退出后重新开启。'.encode()
		cmd = b'\x7e\x05\x01\x00'	#cmd[2] = len(cmdMsg) + 1
		sockTcp.send(cmd+cmdMsg)
		sockTcp.close()
	except Exception as e:
		logger.warning(repr(e))

def initLogger(logPath, logName, logLevel):
	global logger
	logger = logging.getLogger(logName)
	logger.setLevel(logging.DEBUG)

	handlerConsole = logging.StreamHandler()
	handlerConsole.setLevel(logging.DEBUG)
	logPath = os.path.join(logPath, logName)
	handlerFile = logging.handlers.TimedRotatingFileHandler(filename=logPath, when='D', interval=1, encoding="utf-8")
	handlerFile.suffix = "%y-%m-%d.log"
	logLev = logging.INFO
	if logLevel == 5:
		logLev = logging.DEBUG
	elif logLevel == 3:
		logLev = logging.WARNING
	elif logLevel == 2:
		logLev = logging.ERROR
	handlerFile.setLevel(logLev)
	logFormat = logging.Formatter("%(asctime)s %(levelname)s %(message)s", datefmt="%H:%M:%S")
	handlerConsole.setFormatter(logFormat)
	handlerFile.setFormatter(logFormat)

	logger.addHandler(handlerConsole)
	logger.addHandler(handlerFile)

if __name__ == '__main__':
	initLogger(TMP_PATH, 'dyc-up', 5)
	logger.info('start clientupdate')
	if os.sep == '/':
		with os.popen('who') as pi:
			buf = pi.read()
			if len(buf) > 0:
				curUser = buf.split()[0]
				logger.info('curUser:' + curUser)
				updateConfig()

	while True:
		updateClient()
		#daemonClientApp()
		time.sleep(60)

