import os, sys, json, uuid, requests
from enum import Enum

class TaskState(Enum):
	OK = 0
	WAIT = 1
	PROGRESS = 2
	SUSPICIOUS = -2
	CRITICAL = -3
	ERROR = -4

class FileCheck():
	'''文件检查。测试：连接失败、提交多个文件、查询状态、部分失败、修改密级、增删文件并重新提交；通过、违规、可疑 '''

	def __init__(self, sysInfo):
		self.cfg = {} # name,url,access,secret...
		for item in sysInfo.split(';'):
			kv = item.split('=')
			if len(kv) > 1:
				self.cfg[kv[0]] = kv[1]
		self.__secretName = None
		self.__taskUid = None # flowId
		self.__progress = 0
		self.__state = TaskState.ERROR.value
		self.dicFileList = {} # filePath : 'chkId state detailCount \n first_detail_report'
		self.header = {} # http header

	def startCheck(self, taskInfo, fileList:list) -> str:
		'''ret: None=start-OK, empty=re-start OK, or error message '''
		print(self.cfg['name'], 'startCheck', taskInfo['SecretLevelName'])
		self.__userName = taskInfo['UserName']
		self.__docName = taskInfo['DocumentName']
		self.__progress = 0
		if self.__secretName != taskInfo['SecretLevelName']: # secret changed, clear all
			self.finishCheck(False)
		self.__secretName = taskInfo['SecretLevelName']
		if self.__taskUid == None:
			self.dicFileList.clear()

		nFileDel = 0
		for f1 in list(self.dicFileList):
			if f1 not in fileList: # remove file and re-check
				print('remove', f1)
				del self.dicFileList[f1]
				nFileDel += 1
		if nFileDel > 0:
			self.finishCheck(False)

		nFileAdd = 0
		for f1 in fileList:
			if f1 not in self.dicFileList and os.path.isfile(f1): # add file and re-check
				print('add', f1)
				self.dicFileList[f1] = None
				nFileAdd += 1

		ret = None
		if self.cfg['name'] == 'shengkewo':
			SEC_NAME = {'公开':'gongkai', '内部':'neibu', '秘密':'mimi', '机密':'jimi', }
			secretLevel = SEC_NAME[self.__secretName] if self.__secretName in SEC_NAME else 'gongkai'
			self.cfg['secname'] = secretLevel
			self.header = {'X-ACCESS-KEY': self.cfg['access'], 'X-SECRET-KEY': self.cfg['secret']}
			EXT_LIST = ['doc','docx','wps','odt','ott','dot','dotx','wpt','xls','xlsx','et','ods','ots','xlt','xlts','ett','ppt','pptx','dps','odp','otp','pot','potx','wpt','dpt','pdf','ofd','rtf','txt','tif','tiff','jpg','jpeg','bmp','gif','png','dib','dwg','rar','zip','tar','tgz','7z','iso','gz','bz','lzma','bz2','tgz','eml','msg','pst','htm','html','xhtml','tml','mht','mhtml','xml','js','css']

			requests.packages.urllib3.disable_warnings()
			if self.__taskUid == None:	# create workflow
				param = {'type': self.cfg['type'], 'secretLevel': secretLevel, 'promoter': self.__userName, 'extraProps':{'name':self.__docName}}
				res = requests.post(self.cfg['url']+'/workflows', json=param, headers=self.header, verify=False)
				if res.status_code != 200:
					return '创建工作流失败 '+str(res.status_code)

				#{'id': '86b3b391-85a4-419a-9149-3c7000fc4a88', 'type': 'dake', 'state': 'running', 'promoter': 'test1', 'approvers': [], 'stakeholders': [], 'sourceUrl': None, 'extraProps': {}}
				print('workflow:', res.text)
				flow = json.loads(res.text)
				self.__taskUid = flow['id']

			attachUrl = self.cfg['url']+'/workflows/'+self.__taskUid+'/attachments'
			for f1 in self.dicFileList:
				if self.dicFileList[f1] != None: # already uploaded
					continue
				ext = os.path.splitext(f1)[1] # '' or .zip
				if ext != '' and ext[1:] not in EXT_LIST:
					continue

				fp = open(f1, 'rb')
				fName = f1[f1.rfind(os.sep)+1:]
				res = requests.post(url=attachUrl, headers=self.header, files={'data': fp}, data={'name':fName}, verify=False)
				fp.close()
				if res.status_code != 200:
					return '上传文件失败 '+str(res.status_code)

				#{'id': 'b3fbbe1b-8e72-48a3-bdf8-e066dab65085', 'workflowId': '86b3b391-85a4-419a-9149-3c7000fc4a88', 'name': '测试文件1.doc', 'inspectState': 'running'}
				print(res.text)
				attachInfo = json.loads(res.text)
				self.dicFileList[f1] = attachInfo['id'] + ' 1'
			if nFileAdd > 0:
				self.__state = TaskState.WAIT
		elif self.cfg['name'] == 'demo':
			if self.__taskUid == None:
				self.__taskUid = str(uuid.uuid1())
			for f1 in self.dicFileList:
				if self.dicFileList[f1] == None:
					self.dicFileList[f1] = str(uuid.uuid1()) + ' 1'
		else:
			ret = '未识别的系统！'
		if ret == None and nFileAdd == 0 and nFileDel == 0: # no file added and submit again
			ret = '' # SUSPICIOUS
			print(self.__state, self.dicFileList)
			if self.__state <= TaskState.CRITICAL.value: # CRITICAL or ERROR
				for f1,res in self.dicFileList.items():
					if res != None and int(res.split()[1]) <= TaskState.CRITICAL.value:
						ret = '仍包含高危或错误文件'
						break
		return ret

	def getStatus(self, ) -> dict:
		'''ret code: TaskState.XXX.value; status: 简短状态, msg: 检查结果详情'''
		self.__progress += 1
		ret = {'code':TaskState.OK.value, 'status':'', 'msg':None}
		msgUrl = None
		# update status for all files in dicFileList
		if self.cfg['name'] == 'shengkewo':
			resultUrl = self.cfg['url']+'/workflows/'+self.__taskUid+'/inspect-result'
			res = requests.get(resultUrl, params={"workflowid": self.__taskUid}, headers=self.header, verify=False)
			if res.status_code != 200:
				ret['code'] = TaskState.ERROR.value
				ret['msg'] = '查询失败 '+str(res.status_code)
				return ret
			print(res.text)
			#{'result': 'normal', 'description': '工作流检查正常'} running检查中, critical高危命中,suspicious,工作流包含可疑文件; failed,工作流检查失败;
			info = json.loads(res.text)
			ret['status'] = info['description']
			CHECK_STATE = {'normal':0, 'running':2, 'critical':-3, 'suspicious':-2, 'failed': -4}
			ret['code'] = CHECK_STATE[info['result']] if info['result'] in CHECK_STATE else -4
			if info['result'] == 'suspicious' or info['result'] == 'critical':
				msgUrl = self.cfg['url'].replace(':9999',':8444').replace('/v2','')
				msgUrl += '/forward?token=' + self.getJwtToken(self.cfg['secname'])
				msgUrl += '&target=/pages/workflows/'+self.__taskUid

				reportUrl = self.cfg['url']+'/workflows/'+self.__taskUid+'/attachments/'#+attachmentId+'/report'
				print('get report', info['result'], reportUrl)
				for f1,res in self.dicFileList.items():	# todo: get report by attachmentId
					if res == None:
						continue
					url1 = reportUrl + res.split()[0] +'/report'
					param = {'workflowid': self.__taskUid, 'attachmentId':res.split()[0]}
					res = requests.get(url1, headers=self.header, params=param, verify=False)
					if res.status_code != 200:
						print('get report', str(res.status_code))
						self.dicFileList[f1] = self.dicFileList[f1][:-1] + '-4 0\n'
						break
					print(res.text)
					'''{"attachmentId":"82be7ee9-dac3-4c01-a630-00c342fd1472","name":"F:/res.txt","details":[{"name":"会议1","summary":""},{"name":"会议1","summary":"电脑的开<em>机密</em>码是很简单的","description":"res.doc"}]}'''
					report = json.loads(res.text)
					if len(report['details']) == 0 or report['details'][0]['summary'] == None:
						continue
					self.dicFileList[f1] = self.dicFileList[f1][:-1] + str(ret['code']) +' '+str(len(report['details']))+'\n'
					for dt in report['details']:
						if len(dt['summary']) > 0:
							self.dicFileList[f1] += dt['summary'][:255]
							break
					self.dicFileList[f1] += '\n'

		elif self.cfg['name'] == 'demo':
			if self.__progress < 1:
				ret['code'] = TaskState.WAIT.value
				ret['status'] = '排队中……'
			elif self.__progress < 2:
				ret['code'] = TaskState.PROGRESS.value
				ret['status'] = '文档查验……'
				for f1 in self.dicFileList:
					if '\n' in self.dicFileList[f1]: # already checked
						continue
					print('checking', f1)
					if self.__secretName != '秘密' and '秘密' in f1:
						ret['code'] = TaskState.SUSPICIOUS.value
						self.dicFileList[f1] = 'id -2 3\n xxx <em>秘密</em> yyy\n'
					elif self.__secretName != '机密' and '机密' in f1:
						ret['code'] = TaskState.CRITICAL.value
						self.dicFileList[f1] = 'id -3 2\n ..<em>机密<em>...\n'
					elif '错误' in f1:
						ret['code'] = TaskState.ERROR.value
						self.dicFileList[f1] = 'id -4 2\n...\n'
			else: # finished
				ret['code'] = TaskState.OK.value # OK, ERROR #TaskState.SUSPICIOUS.value

		# finished. check status and code(<=0)
		warnCount = 0
		for f1,res in self.dicFileList.items():
			if res == None: # file.ext not supported
				continue
			lst = res.split() # id code count \n msg
			#if int(lst[1]) >= ret['code'] and ret['msg'] != None: # checking or OK
			#	continue
			if ret['code'] >= int(lst[1]):
				ret['code'] = int(lst[1])
				warnCount = int(lst[2]) if len(lst)>2 else warnCount
				ret['msg'] = f1 +' : '+ res[res.find('\n')+1:]

		if ret['code'] == TaskState.CRITICAL.value:
			ret['msg'] += '\n包含以上等违规信息，共计 %d 条，请重新选择密级和文件' % warnCount
		elif ret['code'] == TaskState.SUSPICIOUS.value:
			ret['msg'] += '\n包含以上等可疑信息，共计 %d 条，请确认是否合规，再点击确定提交' % warnCount
		elif ret['code'] == TaskState.ERROR.value:
			ret['msg'] += '\n检查错误'
		self.__state = ret['code']
		if msgUrl != None:
			ret['msg'] = msgUrl
		return ret

	def getResult(self,):
		print('getResult-Report')
		ret = ''
		for f1,res in self.dicFileList.items():
			ret += f1[f1.rfind(os.sep)+1:]
			if res!=None and '\n' in res:
				ret += ' : '+ res[res.find('\n')+1:] +'\n'
		return ret

	def finishCheck(self, succ):
		print('finishCheck', succ)
		if self.cfg['name'] == 'shengkewo' and self.__taskUid != None:
			completeUrl = self.cfg['url']+'/workflows'+'/'+self.__taskUid+ ('/complete' if succ else '/terminate')
			res = requests.get(completeUrl, params={"workflowid": self.__taskUid}, headers=self.header, verify=False)

		self.__taskUid = None
		self.__progress = 0
		self.__state = TaskState.ERROR.value
		for f1 in self.dicFileList:
			self.dicFileList[f1]=None

	def getJwtToken(self, secName):
		import time,base64,hmac,hashlib
		head = {'alg':'HS256','typ':'JWT'}
		ts = int(time.time()) - 10
		payload = {'jti':str(uuid.uuid1()), 'iss':self.cfg['access'], 'sub':self.__userName, 'iat':ts, 'sec':'jimi'}

		first = base64.urlsafe_b64encode(json.dumps(head,separators=(',',':')).encode()).decode().replace('=','')
		second = base64.urlsafe_b64encode(json.dumps(payload,separators=(',',':')).encode()).decode().replace('=','')
		first_second = first+'.'+second
		third = base64.urlsafe_b64encode(hmac.new(self.cfg['secret'].encode(),first_second.encode(),hashlib.sha256).digest()).decode().replace('=','')
		return '.'.join([first,second,third])

