#coding: utf-8

import os, socket, json, time

from cdwriter import CDWriter
from cdwriterwin import CDWriterWin

class CDWriterCop(CDWriter, CDWriterWin):
	ADRS_SVC = ('127.0.0.1', 17008)

	def __init__(self, devName, mainConfig):
		if os.sep == '/':
			CDWriter.__init__(self, devName, mainConfig)
		else:
			CDWriterWin.__init__(self, devName, mainConfig)
		self.__sockUdp = None
		self.discPos = None # Drive or Print

	def sendCmd(self, cmd):
		try:
			print('sendCmd:'+cmd)
			self.__sockUdp.sendto(cmd.encode('utf-8'), self.ADRS_SVC)
			retBuf, addr = self.__sockUdp.recvfrom(256)
			if addr != self.ADRS_SVC:
				return 'error svc address' # won't happen
			retJson = retBuf.decode('utf-8')
			res = json.loads(retJson)
		except Exception as e:
			return repr(e)
		if res['Ret'] != 0:
			return res['Msg']
		else:
			return None

	def initDevice(self, devAddress, totalCopy):
		adrs = '%s:%d' % (self.ADRS_SVC[0], self.ADRS_SVC[1])
		if os.sep == '/':
			cmd = 'netstat -anp|grep %s' % adrs
		else:
			cmd = 'netstat -ano|find "%s"' % adrs
		with os.popen(cmd) as pipeConsole:
			outData = pipeConsole.read()
		if adrs not in outData:
			return '无法连接设备服务 ' + cmd + outData

		self.__sockUdp = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
		self.__sockUdp.settimeout(90)
		#self.__sockUdp.bind(('127.0.0.1', 17010))
		if os.sep == '/':
			errMsg = CDWriter.initDevice(self, devAddress, totalCopy)
		else:
			errMsg = CDWriterWin.initDevice(self, devAddress, totalCopy)
		if errMsg != None: # cdrskin dev=/dev/sr0 -checkdrive
			return errMsg
		return self.sendCmd('{"Function":"InitDevice", "Param":%d}' % self.totalCopy)

	def beginTask(self, dataFile, cdType, nCopy, listDiscImage):
		return None

	def prepareMedia(self, sizeM, cdTypeNum):
		errMsg = self.sendCmd('{"Function":"PrepareMedia", "Param":0}')
		if errMsg != None:
			return errMsg
		self.discPos = 'Drive'
		if os.sep == '/':
			errMsg = CDWriter.prepareMedia(self, sizeM, cdTypeNum)
		else:
			errMsg = CDWriterWin.prepareMedia(self, sizeM, cdTypeNum)
		return errMsg

	def printDisk(self, discImg):
		self.discPos = 'Print'
		return self.sendCmd('{"Function":"PrintDisk", "Param":"%s"}' % discImg)

	def writeData(self, dataFile):
		if self.discPos == 'Print':
			errMsg = self.sendCmd('{"Function":"PrepareMedia", "Param":1}')
			if errMsg != None:
				return errMsg
			self.discPos = 'Drive'
		time.sleep(30) # wait CDDrive
		if os.sep == '/':
			errMsg = CDWriter.writeData(self, dataFile)
		else:
			errMsg = CDWriterWin.writeData(self, dataFile)
		return errMsg

	def completeMedia(self):
		param = 1 if self.discPos == 'Print' else 0
		self.discPos = None
		return self.sendCmd('{"Function":"CompleteMedia", "Param":%d}' % param)

	def endTask(self):
		return None

	def freeDevice(self, succ):
		errMsg = self.sendCmd('{"Function":"FreeDevice", "Param":%d}' % succ)
		if self.__sockUdp != None:
			self.__sockUdp.close()
		return errMsg

