#coding: utf-8
import sys, os, time, ctypes
from ctypes import create_string_buffer,byref,c_ulong
from PyQt5 import QtCore

sys.path.append(os.path.join(os.path.dirname(__file__), '../common'))
from cdwriterwin import CDWriterWin
import myutil

class CDWriterPrimera(CDWriterWin):
	signalUI = QtCore.pyqtSignal(int, str)

	@staticmethod
	def enumDevice(devInfo):
		dictDev = {'PrinterName':'', 'CDWriterType':1, 'pbyUniqueID':'', 'IsMonitored':''}
		cdw = CDWriterPrimera('', None)
		errMsg = cdw.initDevice(None, 0)
		if errMsg != None:
			return errMsg
		dictDev['PrinterName'] = myutil.host['Name']+'('+cdw.cdwName+')'
		dictDev['pbyUniqueID'] = cdw.devAddress
		return [dictDev]

	def __init__(self, devName, mainConfig):
		super().__init__(devName, mainConfig)
		self.cdwName = devName
		self.configJson = mainConfig
		self.lib = None
		self.hRobot = c_ulong(0)
		self.hDrive = c_ulong(0)
		self.devAddress = None
		self.discPos = None # Drive or Print

	def __getErrorMsg(self, errCode) -> str:
		buf = create_string_buffer(512)
		self.lib.PTRobot_GetErrorString(self.hRobot, errCode, byref(buf),256, 6)
		return str(errCode)+ ':' +buf.raw.decode('utf-16', 'ignore').strip('\0')

	def __waitCmd(self):
		'''wait command finish and return 0 or errorCode '''
		buf = create_string_buffer(128)
		nState = 1 # 0=idle,1=busy,2=error
		nError = 0
		while nState == 1:
			time.sleep(2)
			ret = self.lib.PTRobot_GetRobotStatus(self.hRobot, byref(buf)) # dwSystemState,dwSystemError,dwCurrColorSpits,dwCurrBlackSpits,dwFullColorSpits,dwFullBlackSpits
			nState = buf.raw[0]
			nError = buf.raw[4]
			print(nState)
			if nState == 2 and nError != 0:
				return nError
			if nState == 0:
				break
		return 0

	def initDevice(self, devAddress, totalCopy):
		try:
			self.lib = ctypes.WinDLL(os.path.join(os.path.dirname(__file__), 'PTRobot.dll'))
		except Exception as e:
			return repr(e)
		ret = self.lib.PTRobot_SetupDebugging('c:\\dbg.log'.encode('utf-16-le'), c_ulong(4), 'c:\\trc.log'.encode('utf-16-le'))
		ret = self.lib.PTRobot_Initialize()
		if ret != 0:
			return self.__getErrorMsg(ret)
		ret = self.lib.PTRobot_SetApplicationID(c_ulong(0))
		nDev = c_ulong(4)
		HandleArray = c_ulong * nDev.value
		arRobot = HandleArray(0)
		ret = self.lib.PTRobot_EnumRobots(byref(arRobot), byref(nDev))
		print('EnumRobots', ret, nDev.value, arRobot[0])
		if ret != 0 or nDev.value < 1:
			return self.__getErrorMsg(ret)
		self.hRobot = arRobot[0]

		ret = self.lib.PTRobot_EnumDrives(self.hRobot, byref(self.hDrive), byref(nDev))
		buf = create_string_buffer(512)
		ret = self.lib.PTRobot_GetDriveInfo(self.hDrive, byref(buf))
		drvName = buf.raw[4:260].decode('utf-16').strip('\0')
		drvName = drvName.replace('Optiarc ', '').replace('  on USB 1.0', '') # 4050: Optiarc DVD RW AD-5280S  on USB 1.0
		drvName = drvName.replace('LITEON ', '').replace('  on USB 2.0', '') # SE3: DH-16AFSH PREMM1  on USB 2.0
		print('PTRobot_GetDriveInfo', drvName)

		if devAddress == None: # enumDevice
			listCDDrv = CDWriterWin.enumDevice(None)
			print(listCDDrv)
			for cddrv in listCDDrv:
				if drvName in cddrv['PrinterName']:
					self.devAddress = cddrv['pbyUniqueID']
					self.cdwName = drvName
		else:
			self.devAddress = devAddress
		print(self.cdwName, self.devAddress)
		if drvName not in self.cdwName:
			return 'CDDrive(%s) mismatch with regname(%s)' % (drvName, self.cdwName)

		#ret = self.lib.PTRobot_OpenCloseDrive(self.hDrive, 1)
		buf = create_string_buffer(128)
		ret = self.lib.PTRobot_GetMediaInfo(self.hDrive, byref(buf))
		if ret == 525: # NO_MEDIA
			ret = 0
		elif ret == 0: # Disc in CDDrive
			ret = self.lib.PTRobot_UnLoadDrive(self.hRobot, self.hDrive, 0)
		if ret != 0:
			return self.__getErrorMsg(ret)
		else:
			ret = self.__waitCmd()
		#ret = self.lib.PTRobot_KillSystemError(self.hRobot, 1)
		super().initDevice(devAddress, totalCopy)
		return None

	def beginTask(self, dataFile, cdType, nCopy, listDiscImage):
		return None

	def prepareMedia(self, sizeM, cdTypeNum):
		ret = self.lib.PTRobot_LoadDrive(self.hRobot, self.hDrive, 0, 0)
		if ret != 0:
			return self.__getErrorMsg(ret)
		else:
			ret = self.__waitCmd()
		self.discPos = 'Drive'
		#buf = create_string_buffer(128)
		#ret = self.lib.PTRobot_GetMediaInfo(self.hDrive, byref(buf)) # WCHAR MediaID[20];tszMediaType[20];
		#mediaID = buf.raw[0:40].decode('utf-16').strip('\0')
		#mediaType = buf.raw[40:80].decode('utf-16').strip('\0')
		#print(mediaID, mediaType)
		return CDWriterWin.prepareMedia(self, sizeM, cdTypeNum)

	def printDisk(self, discImg):
		ret = self.lib.PTRobot_LoadPrinterFromDrive(self.hRobot, self.hDrive)
		if ret != 0:
			return self.__getErrorMsg(ret)
		else:
			ret = self.__waitCmd()
		self.discPos = 'Print'
		ret = self.lib.PTRobot_PrintFile(self.hRobot, discImg, 1)
		if ret != 0:
			return self.__getErrorMsg(ret)
		else:
			ret = self.__waitCmd()
		return None

	def writeData(self, dataFile):
		if self.discPos == 'Print':
			ret = self.lib.PTRobot_LoadDrive(self.hRobot, self.hDrive, 100, 0)
			if ret != 0:
				return self.__getErrorMsg(ret)
			else:
				ret = self.__waitCmd()
			self.discPos = 'Drive'
		return CDWriterWin.writeData(self, dataFile)

	def completeMedia(self):
		if self.discPos == 'Print':
			ret = self.lib.PTRobot_UnLoadPrinter(self.hRobot, 0)
		else:
			ret = self.lib.PTRobot_UnLoadDrive(self.hRobot, self.hDrive, 0)
			ret = self.__waitCmd()
			ret = self.lib.PTRobot_OpenCloseDrive(self.hDrive, 1)
		if ret != 0:
			return self.__getErrorMsg(ret)
		else:
			ret = self.__waitCmd()
		self.discPos = None
		return None

	def endTask(self):
		return None

	def freeDevice(self, succ):
		self.lib.PTRobot_Destroy()
		super().freeDevice(0)
		return None

	def devAct(self, param):
		''' device test: cmd=code '''
		if param[2] == '=' and len(param) < 10:
			nCode = int(param[3:])
		ret = 0
		if param.startswith('lp'):
			ret = self.lib.PTRobot_LoadPrinter(self.hRobot, nCode)
		elif param.startswith('pf'):
			ret = self.lib.PTRobot_PrintFile(self.hRobot, param[3:], 1)
		elif param.startswith('oc'):
			ret = self.lib.PTRobot_OpenCloseDrive(self.hDrive, nCode)
		elif param.startswith('ld'): # drive not closed when pos=100(from printer)
			ret = self.lib.PTRobot_LoadDrive(self.hRobot, self.hDrive, nCode, 0)
		elif param.startswith('ud'): # drive not closed
			ret = self.lib.PTRobot_UnLoadDrive(self.hRobot, self.hDrive, nCode)
		elif param.startswith('pd'): # drive not closed
			ret = self.lib.PTRobot_LoadPrinterFromDrive(self.hRobot, self.hDrive)
		elif param.startswith('up'):
			ret = self.lib.PTRobot_UnLoadPrinter(self.hRobot, nCode)
		elif param.startswith('sa'):
			ret = self.lib.PTRobot_SystemAction(self.hRobot, 2**nCode)
		elif param.startswith('ke'):
			ret = self.lib.PTRobot_KillSystemError(self.hRobot, nCode)
		if ret == 0:
			ret = self.__waitCmd()
		if ret != 0:
			print(self.__getErrorMsg(ret))
		return None

if __name__ == '__main__':
	myutil.getHostIpMac()
	myutil.appPath = os.path.dirname(__file__)
	myutil.initLogger('C:/Windows/Temp', 'test_pmy', 4)
	cdw = CDWriterPrimera('', None)
	errMsg = cdw.initDevice(None, 0)
	if errMsg != None:
		print(errMsg)
		cdw.freeDevice(0)
		exit(0)

	print('退出(x) 抓盘到打印机(lp) 打印图片(pf=d:/1.jpg) 抓盘到光驱(ld) 开关光驱(oc=0/1)')
	print('从光驱抓盘输出(ud) 从光驱抓盘到打印机(pd) 从打印机抓盘输出(up) 动作命令(sa)')
	print('位置 0=auto,1=right,2=left,100=printer,200=reject 如：从右桶抓盘到打印机 lp=1')
	print('动作命令(sa) 0=AlignPrinter,1=IgnoreInkLow,')
	while True:
		inp = input()
		if inp == 'x':
			break
		elif len(inp) > 3:
			cdw.devAct(inp)
	cdw.freeDevice(0)
