import time, os, json, ctypes
from PyQt5 import QtCore
import myutil

class CDWriterXinke(QtCore.QObject):
	signalUI = QtCore.pyqtSignal(int, str)
	DEV_PORT = 6000
	if os.sep == '/':
		#libPath = '/usr/local/bin/XinKeBurnSvc/libPMYBurnSvcSdk.so'
		libPath = '/opt/apps/com.xinkelu.burn/files/XinKeBurnSvc/libPMYBurnSvcSdk.so'
	else:
		libPath = os.path.dirname(__file__) + '/PMYBurnSvcSdk.dll'

	def __init__(self, devName, mainConfig):
		super(QtCore.QObject, self).__init__()
		self.__configJson = mainConfig
		self.__devAddress = None # device ip
		self.totalCopy = 0
		self.indexCopy = 0
		self.taskState = 0
		self.lib = None
		self.clientHandle = ctypes.c_ulong(0)
		self.taskHandle = ctypes.c_ulong(0)
		self.taskId = ctypes.create_string_buffer(128)

	@staticmethod
	def enumDevice(devInfo):
		clientHandle = ctypes.c_ulong(0)
		nSize = ctypes.c_int(4096)
		buf = ctypes.create_string_buffer(nSize.value)
		try:
			lib = ctypes.CDLL(CDWriterXinke.libPath) if os.sep=='/' else ctypes.WinDLL(CDWriterXinke.libPath)
			lib.PMYBS_Init()
			ret = lib.PMYBS_Login(devInfo.encode(), CDWriterXinke.DEV_PORT, ctypes.byref(clientHandle))
			errMsg = None
			if ret == 0:
				ret = lib.PMYBS_GetRobotStatus(clientHandle,  ctypes.byref(buf), ctypes.byref(nSize))
			elif ret == 11000:
				errMsg = '无法连接刻录服务 ' + devInfo
			else:
				errMsg = 'Error: ' + str(ret)
			lib.PMYBS_Logout(clientHandle)
			lib.PMYBS_Destory()
			if errMsg != None:
				return errMsg
			jsTxt = buf.value.decode()
			if 'devices' not in jsTxt or 'drives' not in jsTxt:
				return jsTxt
			jsDev = json.loads(jsTxt)
			print(jsDev)
			if len(jsDev['devices']) == 0:
				return '未连接刻录设备'
			devName = '%s(%s)' % (myutil.host['Name'], jsDev['devices'][0]['drives'][0]['dev_desc'])
		except Exception as e:
			return repr(e)

		return [{'PrinterName':devName, 'pbyUniqueID':devInfo}]

	def initDevice(self, devAddress, totalCopy):
		print('initDevice PMYBS_Init IP=%s, cp=%d' % (devAddress, totalCopy))
		self.__devAddress = devAddress

		self.lib = ctypes.CDLL(CDWriterXinke.libPath) if os.sep=='/' else ctypes.WinDLL(CDWriterXinke.libPath)
		self.lib.PMYBS_Init()
		errMsg = None
		nSize = ctypes.c_int(4096)
		buf = ctypes.create_string_buffer(nSize.value)
		ret = self.lib.PMYBS_Login(devAddress.encode(), CDWriterXinke.DEV_PORT, ctypes.byref(self.clientHandle))
		if ret == 0:
			ret = self.lib.PMYBS_GetRobotStatus(self.clientHandle, ctypes.byref(buf), ctypes.byref(nSize))
		elif ret == 11000:
			errMsg = '无法连接刻录服务 ' + devAddress
		else:
			errMsg = 'Error: ' + str(ret)
		if errMsg != None:
			return errMsg

		jsDev = json.loads(buf.value.decode())
		if len(jsDev['devices']) == 0:
			return '未连接刻录设备'
		dev0 = jsDev['devices'][0]
		nDisc = dev0['bin1_disc_count'] + dev0['bin2_disc_count']
		if dev0['system_error'] != 0: # bin1_disc_count=255 when error
			errMsg = dev0['system_error_string']
		#elif nDisc < totalCopy: # not detect disc_bin yet
		#	errMsg = '空盘数量不足(%s)' % (nDisc)
		return errMsg

	def beginTask(self, dataFile, cdType, nCopy, listDiscImage):
		print('beginTask PMYBS_CreateTask PMYBS_AddTree PMYBS_SetTaskConfig')
		self.signalUI.emit(1, '创建任务和参数')
		ret = self.lib.PMYBS_CreateTask(self.clientHandle, 1, dataFile.encode(), ctypes.byref(self.taskHandle))
		if ret != 0:
			return 'Error:' + str(ret)
		self.lib.PMYBS_AddTree(self.taskHandle, dataFile.encode(), None)
		for i in range(len(listDiscImage)):
			self.lib.PMYBS_SetPrintFile2(self.taskHandle, listDiscImage[i].encode(), None, i)

		self.lib.PMYBS_SetTaskConfig(self.taskHandle, b'copy_count', str(nCopy).encode())
		bclose = str(self.__configJson['CloseMedia']).encode()
		self.lib.PMYBS_SetTaskConfig(self.taskHandle, b'reject_if_not_blank', bclose)
		self.lib.PMYBS_SetTaskConfig(self.taskHandle, b'close_disc', bclose)
		self.lib.PMYBS_SetTaskConfig(self.taskHandle, b'test_burn', str(self.__configJson['CDWriteDummy']).encode())
		self.lib.PMYBS_SetTaskConfig(self.taskHandle, b'delete_files', b'1') # delete after burnt
		#self.lib.PMYBS_SetTaskConfig(self.taskHandle, b'disc_label', dataFile.encode())
		self.lib.PMYBS_SetTaskConfig(self.taskHandle, b'burn_speed', str(abs(self.__configJson['CDWSpeed'])).encode())
		self.lib.PMYBS_SetTaskConfig(self.taskHandle, b'disc_output_mode', b'200') # L+R -> Center
		self.lib.PMYBS_SetTaskConfig(self.taskHandle, b'burn_proof', b'1') # burning on data buffering
		if cdType == 3:
			nType = 4 # BDVD 25G
		elif cdType == 2:
			nType = 2 # DVD 4.7G
		else: # 1
			nType = 1 # CD 700M
		self.lib.PMYBS_SetTaskConfig(self.taskHandle, b'disc_type', str(nType).encode())
		self.totalCopy = nCopy
		self.indexCopy = 0
		self.taskState = 0

		self.signalUI.emit(1, '开始刻录任务')
		ret = self.lib.PMYBS_SubmitTask(self.taskHandle, ctypes.byref(self.taskId))
		if ret != 0:
			return 'Error:' + str(ret)
		print('1st copy, PMYBS_SubmitTask: ' + self.taskId.value.decode())
		time.sleep(3)
		return None

	def prepareMedia(self, sizeM, cdTypeNum):
		return None

	def printDisk(self, discImg):
		return None

	def writeData(self, dataFile):
		self.indexCopy += 1
		nSize = ctypes.c_int(51200)
		buf = ctypes.create_string_buffer(nSize.value)
		nError = 0
		while True:
			ret = self.lib.PMYBS_GetTaskStatus(self.clientHandle, self.taskId.value, ctypes.byref(buf), ctypes.byref(nSize))
			if ret != 0: # 11000 net failure, or service stopped
				nError += 1
				self.signalUI.emit(1, 'Error:' + str(ret))
				if nError > 12:
					self.taskState = -1
					return '查询任务失败 Error:' + str(ret)
				time.sleep(5)
				continue
			jsStatus = json.loads(buf.value.decode())
			#print(jsStatus) #DBG_ONLY

			task = jsStatus['tasks'][0]
			if len(task['burn']['discs']) < self.indexCopy: # not start yet
				time.sleep(2)
				continue
			disc = task['burn']['discs'][self.indexCopy-1]
			if 'file_upload' in task:
				self.signalUI.emit(1, task['file_upload']['status_string']) #接收文件中0% [0 Bytes/179.63 MB]
			elif disc['status_code'] == 2: # 0=Idle, 1=Wait, 2=Burnning, 3=OK, 4=Cancel, 5=Fail
				self.signalUI.emit(1, disc['status_string']) # burning 50%
			elif disc['status_code'] == 3: # current copy finished
				break
			time.sleep(2)
			if task['is_finish']:
				self.signalUI.emit(1, '刻录任务完成')
				break

			ret = self.lib.PMYBS_GetRobotStatus(self.clientHandle, ctypes.byref(buf), ctypes.byref(nSize))
			if ret != 0:
				self.taskState = -1
				return 'Error:' + str(ret)
			jsStatus = json.loads(buf.value.decode())
			errMsg = None
			if len(jsStatus['devices']) == 0:
				errMsg = '连接设备失败'
			elif jsStatus['devices'][0]['system_error'] != 0:
				errMsg = jsStatus['devices'][0]['system_error_string']
			else:
				nError = 0
			if errMsg != None:
				nError += 1
				self.signalUI.emit(1, errMsg)
				if nError > 20:
					self.taskState = -1
					return '错误处理超时，结束任务：'+errMsg
			time.sleep(3)
		return None

	def completeMedia(self):
		print('completeMedia')
		return None

	def endTask(self):
		print('endTask PMYBS_CloseTask')
		if self.taskState < 0:
			self.lib.PMYBS_CancleTaskById(self.clientHandle, ctypes.byref(self.taskId))
		self.lib.PMYBS_CloseTask(self.taskHandle)
		return None

	def freeDevice(self, succ):
		print('freeDevice')
		self.lib.PMYBS_Logout(self.clientHandle)
		self.lib.PMYBS_Destory()
		return None

