#coding: utf-8

import sys, os, socket, time, json, enum
from PyQt5 import QtCore, QtGui, QtWidgets
import ui_sbpdiag
from setparam import SetParamDlg

ADRS_SVC = ('127.0.0.1', 17008)

class DevParam(enum.Enum):
	UserDataState=0
	NULL_DISK_1_HIGH=1
	FINISH_DISK_1_HIGH=2
	PRINT_DISK_1_HIGH=3
	CD_1_HIGH=4
	PRINT_X_START=5
	PRINT_Y_START=6
	PRINT_TRY_OUT_POS=7
	FAN_SPEED=8
	FINISH_SPEED=9
	FINISH_REST_SPEED=10
	MECH_SPEED=11
	MECH_REST_SPEED=12
	X_SPEED=13
	Y_SPEED=14
	X_REST_SPEED=15
	Y_REST_SPEED=16
	Y_Cartridge_Pos=17
	VT=18
	DA=19
	Version=20
	SerialNum=21
	Block_Cartridge_Pos=22
	FnFunc=23

class SbpDiagWindow(QtWidgets.QMainWindow):
	def __init__(self):
		super().__init__()
		self.ui = ui_sbpdiag.Ui_SbpDiag()
		self.ui.setupUi(self)
		self.devParam = None

		self.sockUdp = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
		self.sockUdp.settimeout(10)
		self.sockUdp.bind(('127.0.0.1', 17010))

		self.ui.btnPrintCDBmp.clicked.connect(self.onPrintCDBmp)
		self.ui.btnSetParam.clicked.connect(self.onSetParam)
		self.ui.btnDetectStoreTray.clicked.connect(self.onDetectStoreTray)
		self.ui.btnDetectDiscBox.clicked.connect(self.onDetectDiscBox)

		self.ui.btnInitDevice.clicked.connect(lambda evt: self.sendCmd('StartInitDevice', None))
		self.ui.btnClearError.clicked.connect(lambda evt: self.sendCmd('ClearError', None))
		self.ui.btnCleanCartridge.clicked.connect(lambda evt: self.sendCmd('CleanCartridge', None))
		self.ui.btnFetchBlankDisc.clicked.connect(lambda evt: self.sendCmd('FetchBlankDisc', None))
		self.ui.btnEjectCDDrive.clicked.connect(lambda evt: self.sendCmd('EjectCDDrive', None))
		self.ui.btnFetchDiscFromCDDrive.clicked.connect(lambda evt: self.sendCmd('FetchDiscFromCDDrive', None))
		self.ui.btnLoadCDDrive.clicked.connect(lambda evt: self.sendCmd('LoadCDDrive', None))
		self.ui.btnPutDiscInCDDrive.clicked.connect(lambda evt: self.sendCmd('PutDiscInCDDrive', None))
		self.ui.btnEjectStoreTray.clicked.connect(lambda evt: self.sendCmd('EjectStoreTray', None))
		self.ui.btnEjectOutStoreTray.clicked.connect(lambda evt: self.sendCmd('EjectOutStoreTray', None))
		self.ui.btnWithdrawStoreTray.clicked.connect(lambda evt: self.sendCmd('WithdrawStoreTray', None))
		self.ui.btnWithdrawPrintTray.clicked.connect(lambda evt: self.sendCmd('WithdrawPrintTray', None))
		self.ui.btnEjectPrintTray.clicked.connect(lambda evt: self.sendCmd('EjectPrintTray', None))
		self.ui.btnTurnOnFan.clicked.connect(lambda evt: self.sendCmd('TurnOnFan', None))
		self.ui.btnTurnOffFan.clicked.connect(lambda evt: self.sendCmd('TurnOffFan', None))
		self.ui.btnOpenDoor.clicked.connect(lambda evt: self.sendCmd('OpenDoor', None))
		self.ui.btnCloseDoor.clicked.connect(lambda evt: self.sendCmd('CloseDoor', None))
		self.ui.btnFetchDiscFromPrintTray.clicked.connect(lambda evt: self.sendCmd('FetchDiscFromPrintTray', None))
		self.ui.btnPutDiscOnPrintTray.clicked.connect(lambda evt: self.sendCmd('PutDiscOnPrintTray', None))
		self.ui.btnPutDiscOnStoreTray.clicked.connect(lambda evt: self.sendCmd('PutDiscOnStoreTray', None))
		self.ui.btnEnableFn.clicked.connect(lambda evt: self.sendCmd('FNKeyEnable', None))
		self.show()
		if self.sendCmd('GetDeviceConfig', None) > 0:
			self.ui.edtVersion.setText(self.devParam[20])
			self.ui.edtSerial.setText(self.devParam[21])

	def sendCmd(self, cmdID, param):
		ret = 0
		try:
			print('sendCmd:'+cmdID)
			cmd = '{"Function":"%s"' % cmdID
			if param != None:
				cmd += ',"Param":"%s"}' % param
			else:
				cmd += '}'
			self.sockUdp.sendto(cmd.encode('utf-8'), ADRS_SVC)
			time.sleep(0.1)
			#return 0  DBG_ONLY
			retBuf, addr = self.sockUdp.recvfrom(256)
			print(retBuf, addr) #{"Ret":0, "Msg":null, "Param":"170 23115 12938 ..."}
			if addr != ADRS_SVC:
				return -1 # won't happen
			jsRet = json.loads(retBuf.decode('utf-8'))
			if 'Param' not in jsRet:
				return 0
			if cmdID == 'GetDeviceConfig':
				self.devParam = jsRet['Param'].split()
				#for i in range(len(self.devParam)):
				#	self.devParam[i] = int(self.devParam[i])
				ret = 1
			else: # cmdID == 'DetectStoreTray' or cmdID == 'DetectDiscBox':
				ret = int(jsRet['Param'])
		except Exception as e:
			print(repr(e))
			ret = -2
		return ret

	def onPrintCDBmp(self, event):
		fPath = QtWidgets.QFileDialog.getOpenFileName(self, 'add file', '.')[0]
		if fPath == None or len(fPath) == 0:
			return
		self.sendCmd('PrintDisk', fPath)

	def onDetectStoreTray(self, event):
		n = self.sendCmd('DetectStoreTray', None)
		self.ui.edtStoreTrayNum.setText(str(n))

	def onDetectDiscBox(self, event):
		n = self.sendCmd('DetectDiscBox', None)
		self.ui.edtBlankDiscNum.setText(str(n))

	def onSetParam(self, event):
		dlg = SetParamDlg(self.devParam)
		if dlg.exec() == dlg.Rejected:
			return
		btnid = QtWidgets.QMessageBox.question(None, '?', '确定要修改参数吗？', QtWidgets.QMessageBox.Ok|QtWidgets.QMessageBox.Cancel)
		if btnid == QtWidgets.QMessageBox.Cancel:
			return
		listParam = dlg.getParamList()
		for i in range(4):
			self.devParam[i+1] = listParam[i]
		sParam  = ''
		for parm in self.devParam:
			sParam += parm + ' '
		self.sendCmd('SetDeviceConfig', sParam)
		QtWidgets.QMessageBox.information(None, '!', '参数设置成功')

	def closeEvent(self, event):
		print('closeEvent')
		self.sockUdp.close()

if __name__ == '__main__':
	app = QtWidgets.QApplication(sys.argv)

	adrs = '%s:%d' % (ADRS_SVC[0], ADRS_SVC[1])
	if os.sep == '/':
		cmd = 'netstat -anp|grep %s' % adrs
	else:
		cmd = 'netstat -ano|find "%s"' % adrs
	with os.popen(cmd) as pipe:
		outData = pipe.read()
	if adrs not in outData:
		QtWidgets.QMessageBox.warning(None, '!', '无法连接设备服务 ' + cmd + outData)
		sys.exit(0)

	win = SbpDiagWindow()
	app.exec_()
